@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -285,7 +285,10 @@ SQLRETURN SQLColAttributesW( SQLHSTMT statement_handle,
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
-            statement -> state == STATE_S10 )
+            statement -> state == STATE_S10 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -433,7 +436,7 @@ SQLRETURN SQLColAttributesW( SQLHSTMT statement_handle,
                   case SQL_COLUMN_NAME:
                     if ( SQL_SUCCEEDED( ret ) && character_attribute && as1 )
                     {
-                        ansi_to_unicode_copy( character_attribute, (char*) as1, SQL_NTS, statement -> connection );
+                        ansi_to_unicode_copy( character_attribute, (char*) as1, SQL_NTS, statement -> connection, NULL );
                     }
 
                     if ( as1 ) 
@@ -519,7 +522,7 @@ SQLRETURN SQLColAttributesW( SQLHSTMT statement_handle,
               case SQL_COLUMN_NAME:
                 if ( SQL_SUCCEEDED( ret ) && character_attribute && as1 )
                 {
-                    ansi_to_unicode_copy( character_attribute, (char*) as1, SQL_NTS, statement -> connection );
+                    ansi_to_unicode_copy( character_attribute, (char*) as1, SQL_NTS, statement -> connection, NULL );
                 }
                 if ( as1 ) 
                 {
