@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -233,7 +233,7 @@ static char const rcsid[]= "$RCSfile: __handles.c,v $ $Revision: 1.13 $";
  * valid without the danger of a seg-vio.
  */
 
-static DMHENV enviroment_root;
+static DMHENV environment_root;
 static DMHDBC connection_root;
 static DMHSTMT statement_root;
 static DMHDESC descriptor_root;
@@ -247,24 +247,25 @@ static DMHDESC descriptor_root;
  * We also have a mutex to protect the connection pooling code
  *
  * If compiled with thread support the DM allows four different
- * thread strategies
+ * thread strategies:
  *
- * Level 0 - Only the DM internal structures are protected
- * the driver is assumed to take care of it's self
+ * Level 0 - Only the DM internal structures are protected.
+ * The driver is assumed to take care of itself
  *
- * Level 1 - The driver is protected down to the statement level
- * each statement will be protected, and the same for the connect 
- * level for connect functions, note that descriptors are considered
+ * Level 1 - The driver is protected down to the statement level.
+ * Each statement will be protected, and the same for the connect 
+ * level for connect functions. Note that descriptors are considered
  * equal to statements when it comes to thread protection.
  *
- * Level 2 - The driver is protected at the connection level. only
- * one thread can be in a particular driver at one time
+ * Level 2 - The driver is protected at the connection level. Only
+ * one thread can be in a particular driver at one time.
  *
  * Level 3 - The driver is protected at the env level, only one thing
  * at a time.
  *
- * By default the driver open connections with a lock level of 3, 
- * this can be changed by adding the line
+ * By default the driver opens connections with lock level 0; drivers
+ * are expected to be thread safe now. This can be changed by adding
+ * the line
  *
  * Threading = N
  *
@@ -415,8 +416,8 @@ DMHENV __alloc_env( void )
          * add to list of env handles
          */
 
-        environment -> next_class_list = enviroment_root;
-        enviroment_root = environment;
+        environment -> next_class_list = environment_root;
+        environment_root = environment;
         environment -> type = HENV_MAGIC;
 
         SQLGetPrivateProfileString( "ODBC", "Trace", "No",
@@ -460,10 +461,10 @@ DMHENV __alloc_env( void )
                     LOG_INFO,
                     LOG_INFO, environment -> msg );
         }
-    }
+        setup_error_head( &environment -> error, environment,
+                SQL_HANDLE_ENV );
 
-    setup_error_head( &environment -> error, environment,
-            SQL_HANDLE_ENV );
+    }
 
     mutex_exit( &mutex_lists );
 
@@ -478,10 +479,11 @@ int __validate_env( DMHENV env )
 {
 #ifdef FAST_HANDLE_VALIDATE
 
-    if ( *(( int * ) env ) == HENV_MAGIC )
+    if ( env && *(( int * ) env ) == HENV_MAGIC )
         return 1;
     else
         return 0;
+
 #else
 
     DMHENV ptr;
@@ -489,7 +491,7 @@ int __validate_env( DMHENV env )
 
     mutex_entry( &mutex_lists );
 
-    ptr = enviroment_root;
+    ptr = environment_root;
 
     while( ptr )
     {
@@ -520,7 +522,7 @@ void __release_env( DMHENV environment )
 
     mutex_entry( &mutex_lists );
 
-    ptr = enviroment_root;
+    ptr = environment_root;
 
     while( ptr )
     {
@@ -540,7 +542,7 @@ void __release_env( DMHENV environment )
         }
         else
         {
-            enviroment_root = ptr -> next_class_list;
+            environment_root = ptr -> next_class_list;
         }
     }
 
@@ -598,28 +600,29 @@ DMHDBC __alloc_dbc( void )
         connection -> next_class_list = connection_root;
         connection_root = connection;
         connection -> type = HDBC_MAGIC;
-    }
 
-    setup_error_head( &connection -> error, connection,
-            SQL_HANDLE_DBC );
+        setup_error_head( &connection -> error, connection,
+                SQL_HANDLE_DBC );
 
 #ifdef HAVE_LIBPTH
-    pth_mutex_init( &connection -> mutex );
-    /*
-     * for the moment protect on a environment level
-     */
-    connection -> protection_level = TS_LEVEL3;
+        pth_mutex_init( &connection -> mutex );
+        /*
+         * for the moment protect at the environment level
+         */
+        connection -> protection_level = TS_LEVEL3;
 #elif HAVE_LIBPTHREAD
-    pthread_mutex_init( &connection -> mutex, NULL );
-    /*
-     * for the moment protect on a environment level
-     */
-    connection -> protection_level = TS_LEVEL3;
+        pthread_mutex_init( &connection -> mutex, NULL );
+        /*
+         * for the moment protect at the environment level
+         */
+        connection -> protection_level = TS_LEVEL3;
 #elif HAVE_LIBTHREAD
-    mutex_init( &connection -> mutex, USYNC_THREAD, NULL );
-    connection -> protection_level = TS_LEVEL3;
+        mutex_init( &connection -> mutex, USYNC_THREAD, NULL );
+        connection -> protection_level = TS_LEVEL3;
 #endif
 
+    }
+
     mutex_exit( &mutex_lists );
 
     return connection;
@@ -671,7 +674,8 @@ void dbc_change_thread_support( DMHDBC connection, int level )
 int __validate_dbc( DMHDBC connection )
 {
 #ifdef FAST_HANDLE_VALIDATE
-    if ( *(( int * ) connection ) == HDBC_MAGIC )
+
+    if ( connection && *(( int * ) connection ) == HDBC_MAGIC )
         return 1;
     else
         return 0;
@@ -757,15 +761,6 @@ void __release_dbc( DMHDBC connection )
     mutex_exit( &mutex_lists );
 }
 
-/*
- *  get the statement root, for use in SQLEndTran and SQLTransact
- */
-
-DMHSTMT __get_stmt_root( void )
-{
-    return statement_root;
-}
-
 /*
  * allocate and register a statement handle
  */
@@ -785,26 +780,151 @@ DMHSTMT __alloc_stmt( void )
          */
 
         statement -> next_class_list = statement_root;
+#ifdef FAST_HANDLE_VALIDATE
+        if ( statement_root )
+        {
+            statement_root -> prev_class_list = statement;
+        }
+#endif    
         statement_root = statement;
         statement -> type = HSTMT_MAGIC;
-    }
 
-    setup_error_head( &statement -> error, statement,
-            SQL_HANDLE_STMT );
+        setup_error_head( &statement -> error, statement,
+                SQL_HANDLE_STMT );
 
 #ifdef HAVE_LIBPTH
-    pth_mutex_init( &statement -> mutex );
+        pth_mutex_init( &statement -> mutex );
 #elif HAVE_LIBPTHREAD
-    pthread_mutex_init( &statement -> mutex, NULL );
+        pthread_mutex_init( &statement -> mutex, NULL );
 #elif HAVE_LIBTHREAD
-    mutex_init( &statement -> mutex, USYNC_THREAD, NULL );
+        mutex_init( &statement -> mutex, USYNC_THREAD, NULL );
 #endif
 
+    }
+
     mutex_exit( &mutex_lists );
 
     return statement;
 }
 
+/*
+ * assigns a statements to the connection
+ */
+
+void __register_stmt ( DMHDBC connection, DMHSTMT statement )
+{
+    mutex_entry( &mutex_lists );
+
+    connection -> statement_count ++;
+    statement -> connection = connection;
+#ifdef FAST_HANDLE_VALIDATE
+    statement -> next_conn_list = connection -> statements;
+    connection -> statements = statement;
+#endif
+    mutex_exit( &mutex_lists );
+}
+
+/*
+ * Sets statement state after commit or rollback transaction
+ */ 
+void __set_stmt_state ( DMHDBC connection, SQLSMALLINT cb_value )
+{
+    DMHSTMT         statement;
+    SQLINTEGER stmt_remaining;
+
+    mutex_entry( &mutex_lists );
+#ifdef FAST_HANDLE_VALIDATE
+    statement      = connection -> statements;
+    while ( statement )
+    {
+        if ( (statement -> state == STATE_S2 ||
+              statement -> state == STATE_S3) &&
+             cb_value == SQL_CB_DELETE )
+        {
+            statement -> state = STATE_S1;
+            statement -> prepared = 0;
+        }
+        else if ( statement -> state == STATE_S4 ||
+              statement -> state == STATE_S5 ||
+              statement -> state == STATE_S6 ||
+              statement -> state == STATE_S7 )
+        {
+            if( !statement -> prepared && 
+                (cb_value == SQL_CB_DELETE ||
+                 cb_value == SQL_CB_CLOSE) )
+            {
+                statement -> state = STATE_S1;
+            }
+            else if( statement -> prepared )
+            {
+                if( cb_value == SQL_CB_DELETE )
+                {
+                    statement -> state = STATE_S1;
+                    statement -> prepared = 0;
+                }
+                else if( cb_value == SQL_CB_CLOSE )
+                {
+                    if ( statement -> state == STATE_S4 )
+                      statement -> state = STATE_S2;
+                    else
+                      statement -> state = STATE_S3;
+                }
+            }
+        }
+        statement = statement -> next_conn_list;
+    }
+#else
+    statement      = statement_root;
+    stmt_remaining = connection -> statement_count;
+
+    while ( statement && stmt_remaining > 0 )
+    {
+        if ( statement -> connection == connection )
+        {
+            if ( (statement -> state == STATE_S2 ||
+                  statement -> state == STATE_S3) &&
+                 cb_value == SQL_CB_DELETE )
+            {
+                statement -> state = STATE_S1;
+                statement -> prepared = 0;
+            }
+            else if ( statement -> state == STATE_S4 ||
+                  statement -> state == STATE_S5 ||
+                  statement -> state == STATE_S6 ||
+                  statement -> state == STATE_S7 )
+            {
+                if( !statement -> prepared && 
+                    (cb_value == SQL_CB_DELETE ||
+                     cb_value == SQL_CB_CLOSE) )
+                {
+                    statement -> state = STATE_S1;
+                }
+                else if( statement -> prepared )
+                {
+                    if( cb_value == SQL_CB_DELETE )
+                    {
+                        statement -> state = STATE_S1;
+                        statement -> prepared = 0;
+                    }
+                    else if( cb_value == SQL_CB_CLOSE )
+                    {
+                        if ( statement -> state == STATE_S4 )
+                          statement -> state = STATE_S2;
+                        else
+                          statement -> state = STATE_S3;
+                    }
+                }
+            }
+
+            stmt_remaining --;
+        }
+
+        statement = statement -> next_class_list;
+    }
+#endif
+    mutex_exit( &mutex_lists );
+}
+
 /*
  * clear all statements on a DBC
  */
@@ -815,8 +935,42 @@ int __clean_stmt_from_dbc( DMHDBC connection )
     int ret = 0;
 
     mutex_entry( &mutex_lists );
+#ifdef FAST_HANDLE_VALIDATE
+    while ( connection -> statements )
+    {
+        ptr  = connection -> statements;
+        last = connection -> statements -> prev_class_list;
+        
+        connection -> statements = ptr -> next_conn_list;
+        if ( last )
+        {
+            last -> next_class_list = ptr -> next_class_list;
+            if ( last -> next_class_list )
+            {
+                last -> next_class_list -> prev_class_list = last;
+            }
+        }
+        else
+        {
+            statement_root = ptr -> next_class_list;
+            if ( statement_root )
+            {
+                statement_root -> prev_class_list = NULL;
+            }
+        }
+        clear_error_head( &ptr -> error );
+
+#ifdef HAVE_LIBPTH
+#elif HAVE_LIBPTHREAD
+        pthread_mutex_destroy( &ptr -> mutex );
+#elif HAVE_LIBTHREAD
+        mutex_destroy( &ptr -> mutex );
+#endif
+        free( ptr );
+    }        
+#else
     last = NULL;
-    ptr = statement_root;
+    ptr  = statement_root;
 
     while( ptr )
     {
@@ -853,31 +1007,92 @@ int __clean_stmt_from_dbc( DMHDBC connection )
             ptr = ptr -> next_class_list;
         }
     }
-
+#endif
     mutex_exit( &mutex_lists );
 
     return ret;
 }
 
+int __check_stmt_from_dbc_v( DMHDBC connection, int statecount, ... )
+{
+    va_list ap;
+    int states[ MAX_STATE_ARGS ];
+    DMHSTMT ptr;
+    int found = 0;
+    int i;
+
+    va_start (ap, statecount);
+    for ( i = 0; i < statecount; i ++ ) {
+        states[ i ] = va_arg (ap, int );
+    }
+    va_end (ap);
+
+    mutex_entry( &mutex_lists );
+#ifdef FAST_HANDLE_VALIDATE
+    ptr = connection -> statements;
+    while( !found && ptr )
+    {
+        for ( i = 0; i < statecount; i ++ ) {
+            if ( ptr -> state == states[ i ] ) {
+                found = 1;
+                break;
+            }
+       }
+    
+        ptr = ptr -> next_conn_list;
+    }
+#else
+    ptr = statement_root;
+    while( !found && ptr )
+    {
+        if ( ptr -> connection == connection )
+        {
+            for ( i = 0; i < statecount; i ++ ) {
+                if ( ptr -> state == states[ i ] ) {
+                    found = 1;
+                    break;
+                }
+            }
+        }
+
+        ptr = ptr -> next_class_list;
+    }
+#endif
+    mutex_exit( &mutex_lists );
+
+    return found;
+}
+
 /*
  * check if any statements on this connection are in a given state
  */
 
 int __check_stmt_from_dbc( DMHDBC connection, int state )
 {
-    DMHSTMT ptr, last;
-    int ret = 0;
+    DMHSTMT ptr;
     int found = 0;
 
     mutex_entry( &mutex_lists );
-    last = NULL;
+#ifdef FAST_HANDLE_VALIDATE
+    ptr = connection -> statements;
+    while( ptr )
+    {
+        if ( ptr -> state == state ) 
+        {
+            found = 1;
+            break;
+        }
+    
+        ptr = ptr -> next_conn_list;
+    }
+#else
     ptr = statement_root;
-
     while( ptr )
     {
         if ( ptr -> connection == connection )
         {
-            if ( ptr -> state == state ) {
+            if ( ptr -> state == state ) 
+            {
                 found = 1;
                 break;
             }
@@ -885,7 +1100,7 @@ int __check_stmt_from_dbc( DMHDBC connection, int state )
 
         ptr = ptr -> next_class_list;
     }
-
+#endif
     mutex_exit( &mutex_lists );
 
     return found;
@@ -894,21 +1109,36 @@ int __check_stmt_from_dbc( DMHDBC connection, int state )
 int __check_stmt_from_desc( DMHDESC desc, int state )
 {
     DMHDBC connection;
-    DMHSTMT ptr, last;
-    int ret = 0;
+    DMHSTMT ptr;
     int found = 0;
 
     mutex_entry( &mutex_lists );
-    last = NULL;
-    ptr = statement_root;
     connection = desc -> connection;
-
+#ifdef FAST_HANDLE_VALIDATE
+    ptr = connection -> statements;
+    while( ptr )
+    {
+        if ( ptr -> ipd == desc || ptr -> ird == desc || ptr -> apd == desc || ptr -> ard == desc ) 
+        {
+            if ( ptr -> state == state ) 
+            {
+                found = 1;
+                break;
+            }
+        }
+    
+        ptr = ptr -> next_conn_list;
+    }
+#else
+    ptr = statement_root;
     while( ptr )
     {
         if ( ptr -> connection == connection )
         {
-            if ( ptr -> ipd == desc || ptr -> ird == desc || ptr -> apd == desc || ptr -> ard == desc ) {
-                if ( ptr -> state == state ) {
+            if ( ptr -> ipd == desc || ptr -> ird == desc || ptr -> apd == desc || ptr -> ard == desc ) 
+            {
+                if ( ptr -> state == state ) 
+                {
                     found = 1;
                     break;
                 }
@@ -917,7 +1147,7 @@ int __check_stmt_from_desc( DMHDESC desc, int state )
 
         ptr = ptr -> next_class_list;
     }
-
+#endif
     mutex_exit( &mutex_lists );
 
     return found;
@@ -926,21 +1156,36 @@ int __check_stmt_from_desc( DMHDESC desc, int state )
 int __check_stmt_from_desc_ird( DMHDESC desc, int state )
 {
     DMHDBC connection;
-    DMHSTMT ptr, last;
-    int ret = 0;
+    DMHSTMT ptr;
     int found = 0;
 
     mutex_entry( &mutex_lists );
-    last = NULL;
-    ptr = statement_root;
     connection = desc -> connection;
-
+#ifdef FAST_HANDLE_VALIDATE
+    ptr = connection -> statements;
+    while( ptr )
+    {
+        if ( ptr -> ird == desc ) 
+        {
+            if ( ptr -> state == state ) 
+            {
+                found = 1;
+                break;
+            }
+        }
+    
+        ptr = ptr -> next_conn_list;
+    }
+#else
+    ptr = statement_root;
     while( ptr )
     {
         if ( ptr -> connection == connection )
         {
-            if ( ptr -> ird == desc ) {
-                if ( ptr -> state == state ) {
+            if ( ptr -> ird == desc ) 
+            {
+                if ( ptr -> state == state ) 
+                {
                     found = 1;
                     break;
                 }
@@ -949,7 +1194,7 @@ int __check_stmt_from_desc_ird( DMHDESC desc, int state )
 
         ptr = ptr -> next_class_list;
     }
-
+#endif
     mutex_exit( &mutex_lists );
 
     return found;
@@ -967,7 +1212,7 @@ int __validate_stmt( DMHSTMT statement )
 {
 #ifdef FAST_HANDLE_VALIDATE
 
-    if ( *(( int * ) statement ) == HSTMT_MAGIC )
+    if ( statement && *(( int * ) statement ) == HSTMT_MAGIC )
         return 1;
     else
         return 0;
@@ -1009,7 +1254,48 @@ void __release_stmt( DMHSTMT statement )
     DMHSTMT ptr;
 
     mutex_entry( &mutex_lists );
-
+#ifdef FAST_HANDLE_VALIDATE
+    /*
+     * A check never mind
+     */
+    if ( statement && ( *(( int * ) statement ) == HSTMT_MAGIC ))
+    {
+        ptr  = statement;
+        last = statement->prev_class_list;
+        
+        if ( statement -> connection )
+        {
+            DMHDBC connection = statement -> connection;
+            DMHSTMT conn_last = NULL;
+            DMHSTMT  conn_ptr = connection -> statements;
+            while ( conn_ptr )
+            {
+                if ( statement == conn_ptr )
+                {
+                    break;
+                }
+                conn_last = conn_ptr;
+                conn_ptr  = conn_ptr -> next_conn_list;
+            }
+            if ( conn_ptr )
+            {
+                if ( conn_last )
+                {
+                    conn_last -> next_conn_list = conn_ptr -> next_conn_list;
+                }
+                else
+                {
+                    connection -> statements    = conn_ptr -> next_conn_list;
+                }
+            }
+        }
+    }
+    else
+    {
+        ptr  = NULL;
+        last = NULL;
+    }
+#else
     ptr = statement_root;
 
     while( ptr )
@@ -1021,16 +1307,28 @@ void __release_stmt( DMHSTMT statement )
         last = ptr;
         ptr = ptr -> next_class_list;
     }
-
+#endif
     if ( ptr )
     {
         if ( last )
         {
             last -> next_class_list = ptr -> next_class_list;
+#ifdef FAST_HANDLE_VALIDATE
+            if ( last -> next_class_list )
+            {
+                last -> next_class_list -> prev_class_list = last;
+            }
+#endif            
         }
         else
         {
             statement_root = ptr -> next_class_list;
+#ifdef FAST_HANDLE_VALIDATE
+            if ( statement_root )
+            {
+                statement_root -> prev_class_list = NULL;
+            }
+#endif            
         }
     }
 
@@ -1060,7 +1358,7 @@ void __release_stmt( DMHSTMT statement )
 
 DMHDESC __alloc_desc( void )
 {
-    DMHDESC descriptor = NULL;
+    DMHDESC descriptor;
 
     mutex_entry( &mutex_lists );
 
@@ -1073,20 +1371,25 @@ DMHDESC __alloc_desc( void )
          */
 
         descriptor -> next_class_list = descriptor_root;
+#ifdef FAST_HANDLE_VALIDATE
+        if ( descriptor_root )
+        {
+            descriptor_root -> prev_class_list = descriptor;
+        }
+#endif    
         descriptor_root = descriptor;
         descriptor -> type = HDESC_MAGIC;
-    }
 
-    setup_error_head( &descriptor -> error, descriptor,
-            SQL_HANDLE_DESC );
+        setup_error_head( &descriptor -> error, descriptor, SQL_HANDLE_DESC );
 
 #ifdef HAVE_LIBPTH
-    pth_mutex_init( &descriptor -> mutex );
+        pth_mutex_init( &descriptor -> mutex );
 #elif HAVE_LIBPTHREAD
-    pthread_mutex_init( &descriptor -> mutex, NULL );
+        pthread_mutex_init( &descriptor -> mutex, NULL );
 #elif HAVE_LIBTHREAD
-    mutex_init( &descriptor -> mutex, USYNC_THREAD, NULL );
+        mutex_init( &descriptor -> mutex, USYNC_THREAD, NULL );
 #endif
+    }
 
     mutex_exit( &mutex_lists );
 
@@ -1101,7 +1404,7 @@ int __validate_desc( DMHDESC descriptor )
 {
 #ifdef FAST_HANDLE_VALIDATE
 
-    if ( *(( int * ) descriptor ) == HDESC_MAGIC )
+    if ( descriptor && *(( int * ) descriptor ) == HDESC_MAGIC )
         return 1;
     else
         return 0;
@@ -1153,10 +1456,22 @@ int __clean_desc_from_dbc( DMHDBC connection )
             if ( last )
             {
                 last -> next_class_list = ptr -> next_class_list;
+#ifdef FAST_HANDLE_VALIDATE
+                if ( last -> next_class_list )
+                {
+                    last -> next_class_list -> prev_class_list = last;
+                }
+#endif            
             }
             else
             {
                 descriptor_root = ptr -> next_class_list;
+#ifdef FAST_HANDLE_VALIDATE
+                if ( descriptor_root )
+                {
+                    descriptor_root -> prev_class_list = NULL;
+                }
+#endif            
             }
             clear_error_head( &ptr -> error );
 
@@ -1198,7 +1513,21 @@ void __release_desc( DMHDESC descriptor )
     DMHDESC ptr;
 
     mutex_entry( &mutex_lists );
-
+#ifdef FAST_HANDLE_VALIDATE
+    /*
+     * A check never mind
+     */
+    if ( descriptor && ( *(( int * ) descriptor ) == HDESC_MAGIC ))
+    {
+        ptr  = descriptor;
+        last = descriptor->prev_class_list;
+    }
+    else
+    {
+        ptr  = NULL;
+        last = NULL;
+    }
+#else
     ptr = descriptor_root;
 
     while( ptr )
@@ -1210,16 +1539,29 @@ void __release_desc( DMHDESC descriptor )
         last = ptr;
         ptr = ptr -> next_class_list;
     }
+#endif
 
     if ( ptr )
     {
         if ( last )
         {
             last -> next_class_list = ptr -> next_class_list;
+#ifdef FAST_HANDLE_VALIDATE
+            if ( last -> next_class_list )
+            {
+                last -> next_class_list -> prev_class_list = last;
+            }
+#endif            
         }
         else
         {
             descriptor_root = ptr -> next_class_list;
+#ifdef FAST_HANDLE_VALIDATE
+            if ( descriptor_root )
+            {
+                descriptor_root -> prev_class_list = NULL;
+            }
+#endif            
         }
     }
 
